function [J grad] = nnCostFunction(nn_params, ...
                                   input_layer_size, ...
                                   hidden_layer_size, ...
                                   num_labels, ...
                                   X, y, lambda)
%NNCOSTFUNCTION Implements the neural network cost function for a two layer
%neural network which performs classification
%   [J grad] = NNCOSTFUNCTON(nn_params, hidden_layer_size, num_labels, ...
%   X, y, lambda) computes the cost and gradient of the neural network. The
%   parameters for the neural network are "unrolled" into the vector
%   nn_params and need to be converted back into the weight matrices. 
% 
%   The returned parameter grad should be a "unrolled" vector of the
%   partial derivatives of the neural network.
%

% Reshape nn_params back into the parameters Theta1 and Theta2, the weight matrices
% for our 2 layer neural network
Theta1 = reshape(nn_params(1:hidden_layer_size * (input_layer_size + 1)), ...
                 hidden_layer_size, (input_layer_size + 1));

Theta2 = reshape(nn_params((1 + (hidden_layer_size * (input_layer_size + 1))):end), ...
                 num_labels, (hidden_layer_size + 1));

% Setup some useful variables
m = size(X, 1);
         
% You need to return the following variables correctly 
J = 0;
Theta1_grad = zeros(size(Theta1)); %h*n+1
Theta2_grad = zeros(size(Theta2)); %r*h+1


X = [ones(m, 1) X];
a2 = sigmoid(X * Theta1');
n = size(a2,1);
a2 = [ones(n, 1) a2]; %add bias
h = sigmoid(a2 * Theta2'); %hypothesis

y = (eye(num_labels)(y,:));

Theta11 = Theta1(:,(2:end));
Theta22 = Theta2(:,(2:end));

J = ( (-1.0/m) * sum(sum(y.*log(h) + (1-y).*log(1-h))) ) + ( (lambda/(2.0*m))* (sum(sum(Theta11.^2)) + sum(sum(Theta22.^2)) ));



d3 = h - y; %dimensions are m*r r= no. of output labels, m = training examples.

z2 = X * Theta1'; %m*n+1 * n+1*h = m*h where n=no.of inputs and +1 added for bias
d22 = d3 * Theta2(:,2:end); % m*r * r*h = m*h , bias removed and so no column 1 taken
d2 = d22 .* sigmoidGradient(z2);
Delta1 = d2' * X; % h*m * m*n+1 = h*n+1
Delta2 = d3' * a2;  % r*m * m*h+1 = r*h+1  , 
Theta1_grad = (1.0/m) .* Delta1;
Theta2_grad = (1.0/m) .* Delta2;
Theta1(:,1) = 0;
Theta2(:,1) = 0;

temp1 = (lambda/m) .* Theta1;
temp2 = (lambda/m) .* Theta2;

Theta1_grad = Theta1_grad + temp1;
Theta2_grad = Theta2_grad + temp2;



% Unroll gradients
grad = [Theta1_grad(:) ; Theta2_grad(:)];


end
